#pragma interface
#include "component.h"
#include <Menu.h>

class CheckedMenuItem: public MenuItem {
  String label;
  bool checked;
public:
  CheckedMenuItem(const char *s = 0);
  bool isSep() const;	// true if separator item
  void setLabel(const char *s);
  void setChecked(bool);
  bool isChecked() const { return checked; }
  void enable() { state(ENABLE); }
  void disable() { state(DISABLE); }
  void draw(Window *,const Rect &);
  ~CheckedMenuItem();
};

struct MenuContainer {
  MenuContainer(Menu *);
  void addItem(MenuItem *m);
  void delItem(int i);
  virtual void show(bool) = 0;
  int indexOf(Component *m);
  ~MenuContainer();
protected:
  Menu *menu;
};

class MenuItemComponent: public Component, public CheckedMenuItem {
protected:
  MenuContainer *parent;
public:
  MenuItemComponent();
  void doit(Karet *);
  MenuItem *asMenuItem() { return this; }
  void remoteMethod(int cmd);
  ~MenuItemComponent();
};

class MenuComponent: public MenuItemComponent, public MenuContainer {
  Point origin;
public:
  MenuComponent();
  MenuContainer *asMenu() { return this; }
  void show(bool);
  void draw(Window *w,const Rect &r);
  void doit(Karet *app);
  void remoteMethod(int cmd);
  ~MenuComponent();
};

class MenuBarComponent: public Component, public MenuContainer {
  MenuBarComponent(const MenuBarComponent &);
public:
  MenuBarComponent();
  MenuContainer *asMenu() { return this; }
  void remoteMethod(int cmd);
  void show(bool);
  ~MenuBarComponent();
};
