#pragma implementation
#include <stdio.h>
#include "windraw.h"

WindowContext::WindowContext(): DrawingContext(Rect(0,0,-1,-1)) {
  setframe(new Frame("",""));
  panx = pany = 0;
  active = false;
}

WindowContext::~WindowContext() { }

void WindowContext::setActive(bool f) {
  if (f != active) {
    active = f;
    frame->settype(f ? frame->DOUBLE : frame->SINGLE);
  }
}

void WindowContext::setCaption(const char *s) { frame->set(frame->title,s); }

void WindowContext::setTitle(const char *s) { frame->set(s,frame->desc); }

void WindowContext::setphys(const Rect &r) {
  FrameWin::setphys(r);
  cliprect = Rect(0,0,loc.height() - 1,loc.width() - 1);
}

void WindowContext::draw(const Rect &r) {
  IndirectContext gc(this);
  gc.clipRect(r.moverel(pany - translatey,panx - translatex));
  //gc.translate(-panx,-pany);
  paint(&gc);
}

void WindowContext::writeAttr(int x,int y,int n,const CHTYPE *a) {
  write(y - pany + translatey,x - panx + translatex,n,a);
}

void WindowContext::copyArea(const Rect &r,int dx,int dy) {
  if (dy)
    slide(r.moverel(translatey-pany,translatex-panx),DIR_DOWN,dy);
  if (dx)
    slide(r.moverel(translatey-pany,translatex-panx),DIR_RIGHT,dx);
}

void WindowContext::fillRect(const Rect &r,CHTYPE attr) {
  fill(r.moverel(translatey-pany,translatex-panx),attr);
}

void WindowContext::setCursor(int x,int y) {
  //fprintf(stderr,"curpos(%d,%d)\n",y,x);
  bool moved = false;
  x += translatex;
  y += translatey;
  if (x < panx && x >= 0) 
    panx = x, moved = true;
  if (y < pany && y >= 0)
    pany = y, moved = true;
  int w = loc.width();
  if (x >= panx + w)
    panx = x - w + 1, moved = true;
  int h = loc.height();
  if (y >= pany + h)
    pany = y - h + 1, moved = true;
  if (moved)
    redraw();
  Window::curpos(y - pany,x - panx);
}

int WindowContext::getX() const { return col + panx - translatex; }

int WindowContext::getY() const { return row + pany - translatey; }
